#region using
using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;
#endregion

namespace Cotyounoyume{
	public class BodyMorph
	{
		#region common or UI Properties
		public Atom atom;
		public float bodyMorphClosingTimeOffset = 10f;
		public EBATCore core;
		public bool disableLog;
		public bool nippleErectionOnGlobal = true;
		public bool clitorisErectionOnGlobal = true;
		public bool isBodyMorphActive = false;
		#endregion
		
		#region body morph
		protected Dictionary<string, DAZMorph> Bmorph;
		protected float bodyMorphClosingTime;
		#endregion
		
		public BodyMorph(EBATCore core){
			try{
				atom = core.atom;
				isBodyMorphActive = false;
				this.core = core;
			}catch(Exception e){
				core.loggerE(e);
			}
		}
		
		public void loadSettings(){		
			try{
				nippleErectionOnGlobal = core.settings["General"]["NippleErectionOn"].AsBool;
				clitorisErectionOnGlobal = core.settings["General"]["ClitorisErectionOn"].AsBool;
				bodyMorphClosingTimeOffset = core.settings["General"]["BodyMorphDurationTime"].AsFloat;

				JSONNode jsBmorph = core.settings["BodyMorph"];
				Bmorph = new Dictionary<string, DAZMorph>();
				
				JSONStorable geometry = atom.GetStorableByID("geometry");
				DAZCharacterSelector character = geometry as DAZCharacterSelector;
				GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

				Bmorph["Nipple"] = morphControl.GetMorphByDisplayName(jsBmorph["Nipple"]);
				Bmorph["Clitoris"] = morphControl.GetMorphByDisplayName(jsBmorph["Clitoris"]);
			}catch(Exception e){
				core.loggerE(e);
			}
		}
		
		public void applySettings(){

		}
		
		public void maintenance(){

		}

		#region bodyMorph functions
		public void update(float time){
			try{
				applyBodyMorph();
				bodyMorphClosingTime = time + bodyMorphClosingTimeOffset;
			}catch(Exception e){
				core.loggerE(e);
			}
		}
		
		public void applyBodyMorph(){
			try{
				isBodyMorphActive = true;
				int stage = core.getStage();
				float ratio = (stage > 5) ? 1.0f : stage * 0.2f;
				if(nippleErectionOnGlobal){
					Bmorph["Nipple"].SetValue(ratio);
				}
				if(clitorisErectionOnGlobal){
					Bmorph["Clitoris"].SetValue(ratio);			
				}
			}catch(Exception e){
				core.loggerE(e);
			}
		}
		
		public void stopBodyMorph(){
			try{
				Bmorph["Nipple"].SetValue(0f);
				Bmorph["Clitoris"].SetValue(0f);			
			}catch(Exception e){
				core.loggerE(e);
			}
		}
		
		public void manage(float time){
			try{
				if(isBodyMorphActive && bodyMorphClosingTime < time && core.isDuringOrgasm == false){
					core.logger("bodyMorph effect stop.");
					isBodyMorphActive = false;
					stopBodyMorph();
				}
			}catch(Exception e){
				core.loggerE(e);
			}
		}
		
		#endregion
	}
}