#region using
using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;
using MVR.FileManagementSecure;
#endregion

//namespace MVR.FileManagementSecure{
namespace Cotyounoyume{
	public class Sound
	{
		#region common or UI Properties
		public Atom atom;
		public EBATCore core;
		public bool disableLog;
		
		public bool soundOnGlobal = true;
		public AudioSourceControl headAudioSource;
		public string soundDir;
		public string soundDirDefault;
		public string soundDirDefaultSuffix = "Sounds";
		public string soundDirAlt = "Custom\\Sounds\\ExpressionBlushingAndTears";
		
		public List<NamedAudioClip> breath;
		public List<NamedAudioClip> moan;
		public List<NamedAudioClip> climax;

		public float lastPlayTime;
		#endregion
		
		#region blush parameters
		#endregion

		public Sound(EBATCore core){
			try{
				atom = core.atom;
				this.core = core;
				maintenance();
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void loadSettings(){
			try{
				soundOnGlobal = core.settings["General"]["SoundOn"].AsBool;
				headAudioSource = atom.GetStorableByID("HeadAudioSource") as AudioSourceControl;
				headAudioSource.pitch = core.settings["General"]["Pitch"].AsFloat;
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void applySettings(){
			try{

			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void maintenance(){
			try{
				headAudioSource = atom.GetStorableByID("HeadAudioSource") as AudioSourceControl;
				if(FileManagerSecure.DirectoryExists(soundDirAlt)){
					soundDir = soundDirAlt;
				}else{
					soundDir = core.myPath.Replace("/", "\\") + "\\" + "Sounds";
				}
				lastPlayTime = 0f;
	
				breath = new List<NamedAudioClip>();
				moan   = new List<NamedAudioClip>();
				climax = new List<NamedAudioClip>();
				
				setSounds(breath, soundDir + "\\" + "breath");
				setSounds(moan, soundDir + "\\" + "moan");
				setSounds(climax, soundDir + "\\" + "climax");
			}catch (Exception e){
				core.loggerE(e);
			}
		}

		public void setSounds(List<NamedAudioClip> list, string path){
			try{
				string[] files;
				files = FileManagerSecure.GetFiles(path, "*.wav");
				for(int i = 0; i < files.Length; i++){
					core.logger("setSounds: files[" + i + "] = " + files[i]);

					NamedAudioClip nac = loadSound(files[i]);
					if(nac == null){
						core.logger("nac is null");
					}else{
						list.Add(nac);
						core.logger("nac is added");						
					}
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		//Function taken from VAMDeluxe's code
		public NamedAudioClip loadSound(string path){
			try{
				string localPath = SuperController.singleton.NormalizeLoadPath(path);
				NamedAudioClip existing = URLAudioClipManager.singleton.GetClip(localPath);
				if (existing != null)
				{
					return existing;
				}

				URLAudioClip clip = URLAudioClipManager.singleton.QueueClip(SuperController.singleton.NormalizeMediaPath(path));
				if (clip == null)
				{
					return null;
				}

				NamedAudioClip nac = URLAudioClipManager.singleton.GetClip(clip.uid);
				if (nac == null)
				{
					return null;
				}
				return nac;
			}catch (Exception e){
				core.loggerE(e);
				return null;
			}				
		}
		
		public void playSound(string type, float time){
			try{
				if(soundOnGlobal == false){
					return;
				}
				if(type != "climax" && time < lastPlayTime){
					return;
				}
				int rand;
				int num;
				List<NamedAudioClip> list;
				if(type == "breath"){
					list = breath;
				}else if(type == "moan"){
					list = moan;
				}else if(type == "climax"){
					list = climax;
				}else{
					return;
				}
				rand = UnityEngine.Random.Range(0, list.Count);
				float duration = list[rand].sourceClip.length;
				lastPlayTime = time + duration;
				headAudioSource.PlayNow(list[rand]);
				core.expression.openMouth(time, duration);
				
				core.logger("list[" + rand + "]: " + duration);
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void update(float time){
			try{
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void manage(float time){
			try{
			}catch (Exception e){
				core.loggerE(e);
			}
		}
	}
}
//}